/* Copyright (C) 2002, by Sandor Fazekas and Janos Torok */
/* Last modified: $Date: 2004/02/10 14:57:31 $ $Revision: 1.10 $ */
/* global.h */
#ifndef GLOBAL_H
#define GLOBAL_H

#ifdef _WIN32
#include <windows.h>
#define strcasecmp(s1,s2) _stricmp(s1,s2)
#define strncasecmp(s1,s2,n) _stricmp(s1,s2,n)
/* float to double warning disable */
#pragma warning(disable: 4244 4305)
#endif

#include <GL/gl.h>
#include <GL/glu.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <gtk/gtk.h>

#ifndef M_PI
#define M_PI 3.1415926535897932384626433
#endif

#ifndef S_ISDIR
#define S_ISDIR(f) (S_IFDIR&f)
#endif

#ifndef max
#define max(a,b) (a<b?b:a)
#endif

#ifndef min
#define min(a,b) (a>b?b:a)
#endif

#ifndef sgn
#define sgn(a) (a>=0?1:-1)
#endif

#ifndef fsgn
#define fsgn(a) (a==0.0?0.0:(a>0?1.0:-1.0))
#endif

#define MaxLight 10
#define MaxLightParam 4

typedef struct MGLlight_ {
  GLenum light;				/* Specifies light, eg. GL_LIGHT0... */
  int on;				/* whether light is on */
  GLenum type[MaxLightParam];		/* source type, eg. GL_AMBIENT ... */
  GLfloat param[MaxLightParam][4];	/* parameters for light<=4 */
} MGLlight;

#define MaxLightModel 3
typedef struct MGLlightmodel_ {
  int model;				/* the number of different models */
  GLenum modelname[MaxLightModel];	/* model name */
  GLfloat modelparam[MaxLightModel][4];	/* model parameter */
} MGLlightmodel;

typedef struct MGLtexdetail_ {
  GLenum *target;	/* the texture's target GL_TEXTURE_2D or 1D */
  GLint level;		/* level of detail, usually 0 */
  GLint components;	/* color components, usually 3 */
  GLsizei width,height;	/* ... */
  GLint border;		/* border width 0,1 */
  GLenum format;	/* format of pixel data, eg. GL_RGB */
  GLenum type;		/* type of pixel data, eg. GL_UNSIGNED_BYTE */
  unsigned char *data;	/* pixel data */
} MGLtexdetail;

typedef struct MGLtexture_ {
  int num;		/* Number of textures loaded */
  GLuint *texture;	/* the texture's name, an unsigned int */
  MGLtexdetail *detail;	/* details of the texture */
} MGLtexture;

typedef struct MGLperspective_ {
  GLdouble angle;	/* view angle in degrees */
  GLdouble aspect;	/* aspect ration width/height */
  GLdouble zNear;	/* distance>0 of the viewer and near clipping plane */
  GLdouble zFar;	/* distance>0 of the viewer and far clipping plane */
} MGLperspective;

typedef struct MGLquadratic_ {
  GLUquadricObj *q;	/* quadratics object needed for sphere */
  GLint		sl;	/* slices (longitude) usually =2*st */
  GLint		st;	/* stacks (latitude) */
} MGLquadratic;

/* General parameters of GL: light, texture */
typedef struct MGLsettings_ {
  MGLlight light[MaxLight];	/* light parameters */
  MGLlightmodel lightmodel;	/* light models */
  MGLtexture texture;		/* Texture parameters */
  MGLperspective persp;		/* perspective */
  MGLquadratic quad;		/* quadratic parameters */
  double bg[4];			/* background color */
  double f;			/* position factor */
  double x,y,z;			/* general shift */
  GtkWidget *wx,*wy,*wz;	/* widgets for the latter */
  double phix,phiy,phiz;	/* rotation parameters */
  GtkWidget *wphix,*wphiy,*wphiz;/* widgets for the latter */
  GtkWidget *glarea;		/* gl drawing area */
  GtkWidget *quadstw;		/* widget to print quad resolution */
  double cx,cy;			/* cursor coordinates */
  int button;			/* button on the gl window that is pressed */
  int height,width;		/* size of gl window */
  char *outfile;		/* file for output */
} MGLsettings;

#define MTD_STDOUT	0
#define MTD_FILE	1
#define MTD_NOMOVIE	2
#define MTD_NOSNAP	4
#define MTD_NOFORK	8

#define IMEASUREDTMAX 10

typedef struct ANIM_ {
  int running;			/* wether animation is running */
  int timeouttag;		/* tag for timeout */
  int measuredt[IMEASUREDTMAX];	/* measure last IMEASUREDTMAX points */
  int imeasuredt;		/* pointer of the last */
  int framedt;			/* time delay for frame */
  int adjustframedt;		/* whether framedt is to be adjusted */
  int adjustquadres;		/* whether sphere resolution is adjusted */
  GtkObject *animframeslideradj;/* anim frame slider */
  GtkWidget *animframevalue;	/* text box */
  int ff;			/* fast play */
  int frameskip;		/* number of frames to drop at each time */
  int detach;			/* detach rotation from animation */
  int writerc;			/* wheter to write rc file at the end */
  char *rcfile;			/* filename for rc */
  int depth;			/* Depth of the current cut */
  double keyf;			/* key factor */
  int groupbytex;		/* group by texture and not by infile */
} ANIM;

typedef struct MGLblend_ {
  int enabled;			/* if blending is enabled */
  GLenum sfactor,dfactor;	/* factors, eg. GL_DST_COLOR ... */
} MGLblend;

MGLsettings gl;
ANIM anim;
char *argv0;
int body_alloc;
GtkWidget *ctrlwin;		/* control toplevel window */
GtkWidget *glwin;		/* gl toplevel window */

#endif
