/* Copyright (C) 2002, by Sandor Fazekas and Janos Torok */
/* Last modified: $Date: 2003/12/23 12:57:28 $ $Revision: 1.2 $ */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include "io.h"
#include "global.h"

/* Prototypes */
void io_setfiletype(int);
void Read_Params(FILE *,char *);
void XYZ_f(int);

typedef struct Param_ {
  const char *name;	/* parameter name */
  int type;		/* type */
  void *data;		/* where data is to be stored */
  int is_req;		/* if it is required */
} Param;

/* types */
#define PARI	1	/* int */
#define PARD	2	/* double */
#define PARS	3	/* string */
#define LNUM	4	/* light num */
#define LPAR	5	/* light struct */
#define COLS	6	/* background */

static Param param_list[] = {
  { "filename",		PARS, &io.filename,		0 },
  { "width",		PARI, &gl.width,		0 },
  { "height",		PARI, &gl.height,		0 },
  { "frameskip",	PARI, &anim.frameskip,		0 },
  { "dx",		PARD, &gl.x,			0 },
  { "dy",		PARD, &gl.y,			0 },
  { "dz",		PARD, &gl.z,			0 },
  { "phix",		PARD, &gl.phix,			0 },
  { "phiy",		PARD, &gl.phiy,			0 },
  { "phiz",		PARD, &gl.phiz,			0 },
  { "quadres",		PARI, &gl.quad.st,		0 },
  { "factor",		PARD, &gl.f,			0 },
  { "bg",		COLS, &gl.bg,			0 },
  { "adjustframedt",	PARI, &anim.adjustframedt,	0 },
  { "adjustquadres",	PARI, &anim.adjustquadres,	0 },
  { "framedt",		PARI, &anim.framedt,		0 },
};

#define PARAM_NUM sizeof(param_list)/sizeof(Param)

int Read_rc(char *prg)
{
  FILE *in;
  char *home,*fullname;

  home=getenv("HOME");
  if (home==NULL) home=strdup("");
  fullname=(char *) calloc(strlen(home)+strlen(anim.rcfile)+2,sizeof(char));
  sprintf(fullname,"%s/%s",home,anim.rcfile);
  in=fopen(fullname,"r");
  if (in==NULL) return(0);
  io_setfiletype(0);	/* rc file should be ascii */
  Read_Params(in,prg);

  fclose(in);
  return(1);
}

void Write_rc(void)
{
  int a;
  FILE *out;
  double *dp;
  char *home,*fullname;

  XYZ_f(0);
  home=getenv("HOME");
  if (home==NULL) home=strdup("");
  fullname=(char *) calloc(strlen(home)+strlen(anim.rcfile)+2,sizeof(char));
  sprintf(fullname,"%s/%s",home,anim.rcfile);
  out=fopen(fullname,"w");
  if (out==NULL) return;
  for (a=0;a<PARAM_NUM;a++) {
    fprintf(out,"%s = ",param_list[a].name);
    switch (param_list[a].type) {
      case PARS:
	fprintf(out,"%s\n",*((char **)param_list[a].data));
	break;
      case PARI:
	fprintf(out,"%d\n",*((int *)param_list[a].data));
	break;
      case PARD:
	fprintf(out,"%le\n",*((double *)param_list[a].data));
	break;
      case COLS:
	dp=(double *)(param_list[a].data);
	fprintf(out,"%le %le %le %le\n",*dp,*(dp+1),*(dp+2),*(dp+3));
	break;
    }
  }
  fclose(out);
  XYZ_f(1);
  fprintf(stderr,".mozirc is written\n");
}

int Which_Param(char *p)
{
  int a;

  for (a=0;a<PARAM_NUM;a++) {
    if (strcmp(p,param_list[a].name)==0) {
      return(a);
    }
  }
  return(-1);
}

void Read_Params(FILE *in,char *prg)
{
  int ppos;
  int a;
  char *p,*pp;
  double *dp;

  while (!io.eof(in)) {
    io.getline(in);
    p=io.buff;
    /* remove '\n at the end */
    if (p[strlen(p)-1]=='\n') p[strlen(p)-1]='\0';
    /* remove whitespaces */
    while (isspace(*p)) p++;
    if (*p!='#') {	/* comment */
      pp=p;
      while (!isspace(*pp)) pp++;
      *pp='\0';
      ppos=Which_Param(p);
      if (ppos>=0) {
	p=pp+1;
	while ((isspace(*p)) || (*p=='=')) p++;
	/* now set value */
	switch (param_list[ppos].type) {
	  case PARI:	/* int */
	    *((int *)(param_list[ppos].data))=atoi(p);
	    param_list[ppos].is_req=0;
	    break;
	  case PARD:	/* double */
	    *((double *)(param_list[ppos].data))=atof(p);
	    param_list[ppos].is_req=0;
	    break;
	  case PARS:	/* string no static is assumed */
	    if (*((char **)(param_list[ppos].data))==NULL) {
	      free(*((char **)(param_list[ppos].data)));
	    }
	    *((char **)(param_list[ppos].data))=strdup(p);
	    param_list[ppos].is_req=0;
	    break;
	  case COLS:
	    dp=(double *)(param_list[ppos].data);
	    sscanf(p,"%le %le %le %le",dp,dp+1,dp+2,dp+3);
	    param_list[ppos].is_req=0;
	    break;
	  default:
	    fprintf(stderr,"%s error: unknown data type\n",prg);
	    exit(1);
	}
      } else {
	fprintf(stderr,"%s warning: unknown parameter:\n",prg);
	fprintf(stderr,"> %s\n",p);
      }
    }
  }
  for (a=0;a<PARAM_NUM;a++) {
    if (param_list[a].is_req) {
      fprintf(stderr,"%s error: parameter %s was not initialized\n",
	  prg,param_list[a].name);
      exit(1);
    }
  }
}
