/* Copyright (C) 2002, by Sandor Fazekas and Janos Torok */
/* Last modified: $Date: 2004/01/26 14:50:00 $ $Revision: 1.13 $ */

#include "rpn.h"

#ifdef _WIN32
#define strncasecmp(s1,s2,n) strnicmp(s1,s2,n)
#endif

#ifndef min
#define min(a,b) (a>b?b:a)
#endif

/* global variables */
const char RPN_openbracket[4]="({[";
const char RPN_closebracket[4]=")}]";
const char RPN_binaryoperator[20]="^*/%+->=<&|";
const char RPN_binoporder[20]="01112233344";
const char RPN_unaryoperator[RPN_UONum][5]={ "sin", "cos",
	"tg", "ctg", "asin", "acos", "atg", "actg", "sqrt",
	"sqr","abs", "sgn", "exp", "log", "norm" };
const char RPN_number[12]="0123456789.";
const char RPN_sign[3]="+-";
#define RPN_TimeVarsNum 7
const char RPN_TimeVars[RPN_TimeVarsNum][3]=
		{ "t", "dx", "dy", "dz", "rx", "ry", "rz" };

/* for debug */
void printRPN(RPN *rpnroot)
{
  int a;
  RPN *rp;

  a=0;
  rp=rpnroot;
  while (rp!=NULL) {
    printf("%2d rpn: %d type: %d rank: %d ival: %d dval: %lf\n",
    		a,rp->rpn,rp->type,rp->rank,rp->ival,rp->dval);
    rp=rp->next;
    a++;
  }
}

/* free allocated objects */
void RPN_Free(RPN **rpnroot)
{
  RPN *rp,*rp2;

  rp=*rpnroot;
  while (rp!=NULL) {
    rp2=rp;
    rp=rp->next;
    free(rp2);
  }
}

double fvbinary(double x,double y,int fv)
{
  switch (fv) {
    case 0:
      return(pow(x,y));
      break;
    case 1:
      return(x*y);
      break;
    case 2:
      return(x/y);
      break;
    case 3:
      return(fmod(x,y));
      break;
    case 4:
      return(x+y);
      break;
    case 5:
      return(x-y);
      break;
    case 6:
      return((double)(x>y));
      break;
    case 7:
      return((double)(x==y));
      break;
    case 8:
      return((double)(x<y));
      break;
    case 9:	/* and */
      return((double)((x>0.0)*(y>0.0)));
      break;
    case 10:	/* or */
      return((double)((x>0.0)+(y>0.0)));
      break;
  }
  return(0.0);
}

double fvunary(double x,int fv)
{
  switch (fv) {
    case 0:
      return(sin(x));
      break;
    case 1:
      return(cos(x));
      break;
    case 2:
      return(tan(x));
      break;
    case 3:
      return(1.0/tan(x));
      break;
    case 4:
      if (fabs(x)>1) return(0.0);
      return(asin(x));
      break;
    case 5:
      if (fabs(x)>1) return(0.0);
      return(acos(x));
      break;
    case 6:
      return(atan(x));
      break;
    case 7:
      return(atan(1.0/x));
      break;
    case 8:
      return(sqrt(fabs(x)));
      break;
    case 9:
      return(x*x);
      break;
    case 10:
      return(fabs(x));
      break;
    case 11:	/* sgn */
      if (x==0.0) return(x);
      return((x>0 ? 1.0 : -1.0));
      break;
    case 12:
      return(exp(x));
      break;
    case 13:
      return(log(x));
      break;
  }
  return(0.0);
}

/* Evaluate RPN structure */
double RPN_Evaluate(RPN *rpnroot,double *var)
{
  double *stack;
  int a,sp;
  RPN *rp;
  double val;

  /* count the number of objects */
  rp=rpnroot;
  sp=0;
  while (rp!=NULL) {
    if (rp->type<3) sp++;
    rp=rp->next;
  }
  stack=(double *)calloc(sp,sizeof(double));
  sp=-1;
  rp=rpnroot;
  while (rp!=NULL) {
    switch (rp->type) {
      case RpnT_Const:
	sp++;
	stack[sp]=rp->dval;
        break;
      case RpnT_Var:
	if ((rp->next!=NULL) && (rp->next->type==RpnT_Unary) &&
	    (rp->next->ival==14)) {
	  /* norm must be done here */
	  a=(rp->ival/CEF_SUBPARNUM)*CEF_SUBPARNUM;
	  /* there is no check wheather it has any sense... */
	  sp++;
	  stack[sp]=sqrt(var[a]*var[a]+var[a+1]*var[a+1]+var[a+2]*var[a+2]);
	  rp=rp->next;
	} else {
	  sp++;
	  stack[sp]=var[rp->ival];
	}
        break;
      case RpnT_Unary:
	if (sp>=0) {
	  stack[sp]=fvunary(stack[sp],rp->ival);
	} else {
	  /* error */
	  free(stack);
	  return(0.0);
	}
        break;
      case RpnT_Binary:
	if (sp>0) {
	  sp--;
	  stack[sp]=fvbinary(stack[sp],stack[sp+1],rp->ival);
	} else {
	  /* error */
	  free(stack);
	  return(0.0);
	}
        break;
    }
    rp=rp->next;
  }
  val=stack[0];
  free(stack);
  return(val);
}

/* move object after already converted objects to the right */
void GoOver(RPN *rp,RPN *prp,RPN **rpnroot)
{
  RPN *tmp,*prev;

  if (rp->next==NULL) return;
  if (rp==(*rpnroot)) {
    *rpnroot=(*rpnroot)->next;
  } else {
    prp->next=rp->next;
  }
  prev=rp;tmp=rp->next;
  while ((tmp!=NULL) && (tmp->rpn)) {
    prev=tmp;tmp=tmp->next;
  }
  rp->next=prev->next;
  prev->next=rp;
}

/* operator order: ^ * / % + - = < > & | */
int toRPN(RPN **rpnroot)
{
  int a,rank;
  RPN *rp,*prev;

  rank=0;
  rp=(*rpnroot);
  while (rp!=NULL) {
    if (rp->rank>rank) rank=rp->rank;
    rp=rp->next;
  }
  while (rank>=0) {
    /* main loop */
    prev=rp=(*rpnroot);
    while (rp!=NULL) {
    /* Unary operators are before binary ones */
      if ((rp->rank==rank) && (rp->type==RpnT_Unary) && (!rp->rpn)) {
        rp->rpn=1;
	GoOver(rp,prev,rpnroot);
      }
      prev=rp;
      rp=rp->next;
    }
    /* Finally binary operators */
    for (a=0;a<=RPN_binoporder[strlen(RPN_binaryoperator)-1];a++) {
      prev=rp=(*rpnroot);
      while (rp!=NULL) {
	if ((rp->rank==rank) && (rp->type==RpnT_Binary) &&
	    (RPN_binoporder[rp->ival]==a) && (!rp->rpn)) {
	  rp->rpn=1;
	  GoOver(rp,prev,rpnroot);
	}
	prev=rp;
	rp=rp->next;
      }
    }
    rank--;
  }
  return(1);
}

double tonumber(char **sp,char **error)
{
  double sgn,val,d;

  *error=NULL;
  val=0.0;
  sgn=1.0;
  if (strchr(RPN_sign,**sp)) {
    sgn=(**sp=='-'?-1.0:1.0);
    (*sp)++;
  }
  while (isdigit(**sp)) {
    val=val*10.0+ (**sp-'0');
    (*sp)++;
  }
  if (**sp=='.') {
    (*sp)++;
    d=0.1;
    while (isdigit(**sp)) {
      val=val+d* (**sp-'0');
      d/=10.0;
      (*sp)++;
    }
  }
  if (tolower(**sp)=='e') {
    if (val==0.0) {
      *error=strdup("Invalid number!");
      return(0.0);
    }
    (*sp)++;
    val*=pow(10.0,tonumber(sp,error));
  }
  return(val*sgn);
}

int RPN_decode(char *s,RPN **rpnroot,char **error,int vart)
{
  int a,b,rank,first,l;
  int begin;		/* beginning of a subexpr needed for sign */
  char *sp,ch;
  RPN *rp;

  rank=0;
  begin=first=1;
  sp=s;
  while (*sp) {
    while (isspace(*sp)) sp++;
    if (*sp) {
      if (strchr(RPN_openbracket,*sp)) {
        rank++;
	begin=1;
	sp++;
      } else if (strchr(RPN_closebracket,*sp)) {
        rank--;
	begin=0;
	sp++;
	if (rank<0) {
	  *error=strdup("Too many closing bracket");
	  RPN_Free(rpnroot);
	  return(0);
	}
      } else {	/* not a bracket */
	if (first) {
	  first=0;
	  rp=*rpnroot=(RPN *)malloc(sizeof(RPN));
	} else {
	  rp->next=(RPN *)malloc(sizeof(RPN));
	  rp=rp->next;
	}
	rp->next=NULL;
	rp->rpn=0;
	rp->rank=rank;
	/* check number */
	if ((strchr(RPN_number,*sp)) || ((begin) &&
	    (strchr(RPN_sign,*sp)) && (strchr(RPN_number,*(sp+1))))) {
	  begin=0;
	  rp->dval=tonumber(&sp,error);
	  if (*error!=NULL) {
	    RPN_Free(rpnroot);
	    return(0);
	  }
	  rp->type=RpnT_Const;
	  rp->rpn=1;
	} else if (strchr(RPN_binaryoperator,*sp)) {
	  /* check binary operator */
	  begin=1;
	  rp->ival=strchr(RPN_binaryoperator,*sp)-RPN_binaryoperator;
	  rp->type=RpnT_Binary;
	  sp++;
	} else {
	  /* chek unary operator */
	  for (a=0;a<RPN_UONum;a++) {
	    if ((strncasecmp(RPN_unaryoperator[a],sp,
		  strlen(RPN_unaryoperator[a]))==0) &&
		(!isalnum(*(sp+min(strlen(RPN_unaryoperator[a]),
							strlen(sp)))))) {
		      rp->type=RpnT_Var;
	      break;
	    }
	  }
	  if (a<RPN_UONum) {
	  /* unary */
	    rp->type=RpnT_Unary;
	    rp->ival=a;
	    sp+=strlen(RPN_unaryoperator[a]);
	  } else {
	  /* it must be a variable */
	    rp->type=0;
	    switch (vart) {
	      case RpnVT_Constr :
		for (a=0;a<CEF_PARNUM;a++) {
		  for (b=0;b<CEF_SUBPARNUM;b++) {
		    if ((strlen(cef_parstr[a][b])) &&
			(strncmp(cef_parstr[a][b],sp,
			      strlen(cef_parstr[a][b]))==0) && 
			(!isalnum(*(sp+min(strlen(cef_parstr[a][b]),
							strlen(sp)))))) {
		      rp->type=RpnT_Var;
		      rp->ival=a*CEF_SUBPARNUM+b;
		      rp->rpn=1;
		      sp+=strlen(cef_parstr[a][b]);
		      begin=0;
		      /* exit */
		      a=CEF_PARNUM;
		      break;
		    }
		  }
		}
		if ((rp->type) && (*sp=='t') && (!isalnum(*(sp+1)))) {
		  rp->type=RpnT_Var;
		  rp->ival=CEF_SUBPARNUM*CEF_PARNUM;
		  rp->rpn=1;
		  sp+=strlen(cef_parstr[a][b]);
		  begin=0;
		}
		break;
	      case RpnVT_Time :
		for (a=0;a<RPN_TimeVarsNum;a++) {
		  if ((strncmp(RPN_TimeVars[a],sp,strlen(RPN_TimeVars[a]))==0)
		    && (!isalnum(sp+strlen(cef_parstr[a][b])))) {
		    rp->type=RpnT_Var;
		    rp->ival=a;
		    rp->rpn=1;
		    sp+=strlen(RPN_TimeVars[a]);
		    begin=0;
		  }
		}
	        break;
	    }
	    if (rp->type==0) {
	      l=0;
	      while (isalnum(*(sp+l))) l++;
	      *error=(char *)calloc(strlen("Unkown object: ")+
	      		l+10,sizeof(char));
	      ch=*(sp+l);
	      *(sp+l)='\0';
	      sprintf(*error,"Unkown object: %s",sp);
	      *(sp+l)=ch;
	      RPN_Free(rpnroot);
	      return(0);
	    }
	  }
	}
      }
    }
  }
  if (rank!=0) {
    *error=strdup("Too many opening bracket");
    RPN_Free(rpnroot);
    return(0);
  }
  return(toRPN(rpnroot));
}

